@@ -10,14 +10,163 @@
 
 extern P_fun *P;
 extern G_fun *G;
-extern Gprim_fun *Gprim;
+extern H_fun *H;
+
+static double get2_gam(int n, /* family size */
+		       double *weights,
+		       double *lin, /* x %*% beta for this family */ 
+		       double *yw,
+		       double gam_start);
+
+static double get3_gam(int n, /* family size */
+		       double *weights,
+		       double *lin, /* x %*% beta for this family */ 
+		       double ytot);
+	
+static double gam_fun(double gam, void *extra){
+    int i;
+    double res;
+    Ext_gam *ex;
+
+    ex = extra;
+
+    /* res = ex->ytot; */
+    res = 0.0;
+    for (i = 0; i < ex->n; i++){
+	/* res -= P(ex->lin[i] + gam, 1.0, 1.0);  check !!!!! */
+	res += G(ex->lin[i] + gam, ex->yw[i], ex->weights[i]);
+    }
+    return(res);
+}
+
+static double get_gam(int n, /* family size */
+	       double *lin, /* x %*% beta for this family */ 
+	       int ytot){
+    int i, iter;
+
+    double gam, delta, eps;
+    double *p;
+    double tal, nam, upper, lower;
+
+    p = Calloc(n, double);
+    eps = 0.000001;
+
+    if ((ytot == 0)) error("ytot must be strictly positive.");
+    if ((ytot == n)) error("ytot must be strictly less than n.");
+
+    upper = 0.0;
+    lower = 0.0;
+    for (i = 0; i < n; i++){
+	if (lower < lin[i]) lower = lin[i];
+	if (upper > lin[i]) upper = lin[i];
+    }
+    gam = log((double)ytot / (double)(n - ytot));
+    lower = gam - lower;
+    upper = gam - upper;
+    gam = (upper + lower) * 0.5;
+    if ((upper - lower) < eps) return(gam);
+
+    delta = 1.0;
+    iter = 0;
+    while ((fabs(delta) > eps) && (iter < 100)){
+	iter++;
+	tal = ytot;
+	nam = 0.0;
+	for (i = 0; i < n; i++){
+	    p[i] = P(lin[i] + gam, 1.0, 1.0); /* Change !!!!!! */
+	    tal -= p[i];
+	    nam += p[i] * (1.0 - p[i]);
+	}
+	delta = tal / nam;
+	gam += delta;
+    }
+    tal = 0.0;
+    for (i = 0; i < n; i++) tal += p[i];
+
+    if (iter > 99){
+	Rprintf("gamma = %f\n", gam);
+	Rprintf("ytot = %d\n", ytot); 
+	Rprintf("n = %d\n", n);
+	Rprintf("upper = %f\n", upper);
+	Rprintf("lower = %f\n", lower);
+	for (i = 0; i < n; i++){
+	    Rprintf("lin[%d] = %f\n", i, lin[i]);
+	}
+	error("Too many iterations in [get_gam]");
+    }
+
+    Free(p);
+    return(gam);
+}
+
+static double get2_gam(int n, /* family size */
+		       double *weights,
+		       double *lin, /* x %*% beta for this family */ 
+		       double *yw,
+		       double gam_start){
+
+/* For Binomial families */
+    double ax;
+    double bx;
+    Ext_gam *extra;
+    double Tol;
+    int Maxit;
+    double res;
+    double gam;
+    int i;
+
+    extra = Calloc(1, Ext_gam);
+    /*
+    if (abs(ytot) < 0.0001) error("ytot must be strictly positive.");
+    if (abs(ytot - wtot) < 0.0001) 
+	error("ytot must be strictly less than wtot.");
+    */
+    ax = 0.0;
+    bx = 0.0;
+    for (i = 0; i < n; i++){
+	if (ax < lin[i]) ax = lin[i];
+	if (bx > lin[i]) bx = lin[i];
+    }
+    gam = gam_start;
+    ax = gam - ax;
+    bx = gam - bx; /* Check if not bx = gam + bx!? */
+
+    extra->yw = yw;
+    extra->weights = weights;
+    extra->n = n;
+    extra->lin = lin;
+
+    Tol = 0.0;
+    Maxit = 25;
+    res = GB_zeroin(ax, bx, &gam_fun, extra, &Tol, &Maxit);
+    Free(extra);
+    return(res);
+}
+
+static double get3_gam(int n, /* family size */
+		       double *weights,
+		       double *lin, /* x %*% beta for this family */ 
+		       double ytot){
+
+/* For Poisson family */
+    int j;
+    double denom;
+
+    denom = 0.0;
+    for (j = 0; j < n; j++){
+	denom += weights[j] * exp(lin[j]);
+    }
+
+    return ( log( ytot / denom ) );
+}
+
 
 double bfun(int p, double *b, void *ex){
     int i;
     int j;
     int q; /* No. of families */
     int indx;
-    int ytot;
+    double ytot, wtot;
     double loglik;
 
     double *lin;
@@ -25,7 +174,7 @@ double bfun(int p, double *b, void *ex){
     int *cluster;
     int *ki;
     double **x;
-    int *y;
+    double *y;
     double *offset;
 
     Extb *ext;
@@ -35,7 +184,7 @@ double bfun(int p, double *b, void *ex){
     cluster = ext->cluster;
     ki = ext->ki;
     x = ext->x;
-    y = ext->y;
+    y = ext->yw;
     offset = ext->offset;
 
     lin = ext->x_beta;
@@ -62,34 +211,43 @@ double bfun(int p, double *b, void *ex){
     indx = 0;
     if (ext->family <= 1){ /* binomial family */
 	for (i = 0; i < ext->n_fam; i++){  /* NOT Excluding first family!! */
-	    ytot = 0;
-	    for (j = 0; j < ext->fam_size[i]; j++)
-		ytot += ext->y[indx + j];
-	    if (ytot == 0){
+	    ytot = 0.0;
+	    wtot = 0.0;
+	    for (j = 0; j < ext->fam_size[i]; j++){
+		ytot += ext->yw[indx + j];
+		wtot += ext->weights[indx + j];
+	    }
+	    if (abs(ytot) < 0.001){
 		ext->fam_out[i] = -1;
 		ext->gamma[i] = -1000; /* -inf */
-	    }else if (ytot == ext->fam_size[i]){
+	    }else if (abs(ytot - wtot) < 0.001){
 		ext->fam_out[i] = 1;
 		ext->gamma[i] = 1000;  /* +inf */
 	    }else{
 		ext->fam_out[i] = 0;
-		ext->gamma[i] = get2_gam(ext->fam_size[i], 
+		ext->gamma[i] = get2_gam(ext->fam_size[i],
+					 (ext->weights + indx), 
 					 (lin + indx),
-					 ytot);
+					 (ext->yw + indx),
+					 log( ytot / (wtot - ytot) ));
 	    }
 	    indx += ext->fam_size[i];
 	}
     }else{ /* Poisson; ext->family == 2 */
 	for (i = 1; i < ext->n_fam; i++){ /* Excluding first family!! */
-	    ytot = 0;
-	    for (j = 0; j < ext->fam_size[i]; j++)
-		ytot += ext->y[indx + j];
-	    if (ytot == 0){
+	    ytot = 0.0;
+	    wtot = 0.0;
+	    for (j = 0; j < ext->fam_size[i]; j++){
+		ytot += ext->yw[indx + j];
+		wtot += ext->weights[indx + j];
+	    }
+	    if (abs(ytot) < 0.001){
 		ext->fam_out[i] = -1;
 		ext->gamma[i] = -1000; /* -inf */
 	    }else{
 		ext->fam_out[i] = 0;
 		ext->gamma[i] = get3_gam(ext->fam_size[i],
+					 ext->weights + indx,
 					 (lin + indx),
 					 ytot);
 	    }
@@ -105,7 +263,9 @@ double bfun(int p, double *b, void *ex){
 	if (ext->fam_out[i] == 0){
 	    for (j = 0; j < ext->fam_size[i]; j++){
 		indx++;
-		loglik += log( P(lin[indx] + ext->gamma[i], y[indx]) );
+       
+		loglik += log( P(lin[indx] + ext->gamma[i], 
+				 y[indx], ext->weights[indx]) );
 	    }
 	}else{
 	    indx += ext->fam_size[i];
@@ -141,7 +301,8 @@ void bfun_gr(int n, double *b, double *gr, void *ex){
 	    if (ext->fam_out[i] == 0){
 		for (j = 0; j < ext->fam_size[i]; j++){
 		    indx++;
-		    ext->pred[indx] = P(ext->gamma[i] + lin[indx], 1);
+		    ext->pred[indx] = P(ext->gamma[i] + lin[indx], 1.0, 
+					ext->weights[indx]); /* ????? */
 		}
 	    }else{
 		tmp = (double)(ext->fam_out[i] == 1);
@@ -177,7 +338,7 @@ void bfun_gr(int n, double *b, double *gr, void *ex){
 /* Calculate gr[s]: */
 		for (j = 0; j < ext->fam_size[i]; j++){
 		    indx++;
-		    gr[s] += (ext->y[indx] - pe[indx]) *
+		    gr[s] += (ext->yw[indx] - pe[indx]) *
 			x[indx][s];
 		}
 	    }
@@ -193,28 +354,29 @@ void bfun_hess(int p, double *b, double *hessian, Extb *ext){
 
     int m, s, i, j, indx;
     
-    double *H, *H_fam;
+    double *h, *h_fam;
     double **hess;
     double gam, t1, t2;
     
-    H = Calloc(ext->n, double);
-    H_fam = Calloc(ext->n_fam, double);
+    h = Calloc(ext->n, double);
+    h_fam = Calloc(ext->n_fam, double);
     hess = Calloc(p, double *);
     for (m = 0; m < p; m++){
 	hess[m] = hessian + m * p;
     }
 
-    for (i = 0; i < ext->n; i++) H[i] = 0.0;
+    for (i = 0; i < ext->n; i++) h[i] = 0.0;
 
     indx = -1;
     for (i = 0; i < ext->n_fam; i++){
-	H_fam[i] = 0.0;
+	h_fam[i] = 0.0;
 	if (ext->fam_out[i] == 0){
 	    gam = ext->gamma[i];
 	    for (j = 0; j < ext->fam_size[i]; j++){
 		indx++;
-		H[indx] = Gprim(ext->x_beta[indx] + gam, ext->y[indx]);
-		H_fam[i] += H[indx];
+		h[indx] = H(ext->x_beta[indx] + gam, ext->yw[indx],
+				ext->weights[indx]);
+		h_fam[i] += h[indx];
 	    }
 	}else{
 	    indx += ext->fam_size[i];
@@ -230,7 +392,7 @@ void bfun_hess(int p, double *b, double *hessian, Extb *ext){
     for (m = 0; m < p; m++){
 	for (s = 0; s <= m; s++){
 	    for (i = 0; i < ext->n; i++){
-		hess[m][s] += ext->x[i][m] * ext->x[i][s] * H[i];
+		hess[m][s] += ext->x[i][m] * ext->x[i][s] * h[i];
 	    }
 	    indx = -1;
 	    for (i = 0; i < ext->n_fam; i++){
@@ -239,10 +401,10 @@ void bfun_hess(int p, double *b, double *hessian, Extb *ext){
 		    t2 = 0.0;
 		    for (j = 0; j < ext->fam_size[i]; j++){
 			indx++;
-			t1 += ext->x[indx][m] * H[indx];
-			t2 += ext->x[indx][s] * H[indx];
+			t1 += ext->x[indx][m] * h[indx];
+			t2 += ext->x[indx][s] * h[indx];
 		    }
-		    hess[m][s] -= t1 * t2 / H_fam[i];
+		    hess[m][s] -= t1 * t2 / h_fam[i];
 		}else{
 		    indx += ext->fam_size[i];
 		}
@@ -255,277 +417,7 @@ void bfun_hess(int p, double *b, double *hessian, Extb *ext){
 	    hess[m][s] = hess[s][m];
 	}
     }
-    Free(H_fam);
-    Free(H);
+    Free(h_fam);
+    Free(h);
 }
    
-	
-double gam_fun(double gam, void *extra){
-    int i;
-    double res;
-    Ext_gam *ex;
-
-    ex = extra;
-
-    res = ex->ytot;
-    for (i = 0; i < ex->n; i++){
-	res -= P(ex->lin[i] + gam, 1);
-    }
-    return(res);
-}
-
-double get_gam(int n, /* family size */
-	       double *lin, /* x %*% beta for this family */ 
-	       int ytot){
-    int i, iter;
-
-    double gam, delta, eps;
-    double *p;
-    double tal, nam, upper, lower;
-
-    p = Calloc(n, double);
-    eps = 0.000001;
-
-    if ((ytot == 0)) error("ytot must be strictly positive.");
-    if ((ytot == n)) error("ytot must be strictly less than n.");
-
-    upper = 0.0;
-    lower = 0.0;
-    for (i = 0; i < n; i++){
-	if (lower < lin[i]) lower = lin[i];
-	if (upper > lin[i]) upper = lin[i];
-    }
-    gam = log((double)ytot / (double)(n - ytot));
-    lower = gam - lower;
-    upper = gam - upper;
-    gam = (upper + lower) * 0.5;
-    if ((upper - lower) < eps) return(gam);
-
-    delta = 1.0;
-    iter = 0;
-    while ((fabs(delta) > eps) && (iter < 100)){
-	iter++;
-	tal = ytot;
-	nam = 0.0;
-	for (i = 0; i < n; i++){
-	    p[i] = P(lin[i] + gam, 1);
-	    tal -= p[i];
-	    nam += p[i] * (1.0 - p[i]);
-	}
-	delta = tal / nam;
-	gam += delta;
-    }
-    tal = 0.0;
-    for (i = 0; i < n; i++) tal += p[i];
-
-    if (iter > 99){
-	Rprintf("gamma = %f\n", gam);
-	Rprintf("ytot = %d\n", ytot); 
-	Rprintf("n = %d\n", n);
-	Rprintf("upper = %f\n", upper);
-	Rprintf("lower = %f\n", lower);
-	for (i = 0; i < n; i++){
-	    Rprintf("lin[%d] = %f\n", i, lin[i]);
-	}
-	error("Too many iterations in [get_gam]");
-    }
-
-    Free(p);
-    return(gam);
-}
-
-double get2_gam(int n, /* family size */
-	       double *lin, /* x %*% beta for this family */ 
-	       int ytot){
-    double ax;
-    double bx;
-    Ext_gam *extra;
-    double Tol;
-    int Maxit;
-    double res;
-    double gam;
-    int i;
-
-    extra = Calloc(1, Ext_gam);
-    if ((ytot == 0)) error("ytot must be strictly positive.");
-    if ((ytot == n)) error("ytot must be strictly less than n.");
-
-    ax = 0.0;
-    bx = 0.0;
-    for (i = 0; i < n; i++){
-	if (ax < lin[i]) ax = lin[i];
-	if (bx > lin[i]) bx = lin[i];
-    }
-    gam = log((double)ytot / (double)(n - ytot));
-    ax = gam - ax;
-    bx = gam - bx;
-
-    extra->ytot = ytot;
-    extra->n = n;
-    extra->lin = lin;
-
-    Tol = 0.0;
-    Maxit = 25;
-    res = GB_zeroin(ax, bx, &gam_fun, extra, &Tol, &Maxit);
-    Free(extra);
-    return(res);
-}
-
-double get3_gam(int n, /* family size */
-	       double *lin, /* x %*% beta for this family */ 
-	       int ytot){
-    int j;
-    double denom;
-
-    denom = 0.0;
-    for (j = 0; j < n; j++){
-	denom += exp(lin[j]);
-    }
-
-    return ( log( (double)ytot / denom ) );
-}
-
-
-
-#include "fun.h"
-
-void bnr_opt(int bdim, double *beta, double *loglik, int *mask, 
-	     Extb *ext, double epsilon, int maxit, int trace){
-
-    /* Start a Newton-Raphson thing: */
-    
-    double fstart, fprev;
-    int iter;
-
-    double *db = NULL;
-    double *gr = NULL;
-    double *hess = NULL;
-    void *ex;
-
-    int one = 1;
-    int conver = 0;
-
-    int info = 1;
-    double L1;
-    int i;
-
-    int p, k, m;
-    int true_bdim;
-
-    int *ipiv;
-    double *work;
-    int lwork;
-
-    double rcond;
-    double *det;
-    int job = 11;
-    int pos_def;
-    double *c_hess;
-
-    c_hess = Calloc(bdim * bdim, double);
-    det = Calloc(2, double);
-
-    p = ext->p;
-
-    true_bdim = p; /* p = bdim here!!!!!!!! */
-
-    db = Calloc(bdim, double);
-    ipiv = Calloc(bdim, int);
-    lwork = 11 * bdim;
-    work = Calloc(lwork, double);
-
-    gr = ext->gr;
-    /* hess = ext->hessian; */
-    ex = ext;
-
-    fun2(bdim, beta, loglik, gr, hess, ex);
-    fstart = *loglik;
-    fprev = fstart;
-  
-    for (iter = 0; iter < maxit; iter++){
-	F77_CALL(dcopy)(&true_bdim, gr, &one, db, &one);
-
-	pos_def = 0;
-	while (!pos_def){
-	    F77_CALL(dpoco)(hess, &bdim, &true_bdim, &rcond, work, &info);
-	    
-	    if (info){
-		printf("Hessian not positive definite.\n");
-		printf("info = %d\n", info);
-		if (true_bdim == bdim){
-		    fun2(bdim, beta, loglik, gr, hess, ex);
-		    printf("We try fixing sigma at %f\n", exp(beta[bdim-1]));
-		    true_bdim--;
-		}else{
-		    printf("sigma currently = %f", exp(beta[bdim -1]));
-		    error("Try another start value for sigma.\n");
-		}
-		F77_CALL(dpoco)(hess, &bdim, &true_bdim, 
-				&rcond, work, &info);
-		if (info) error("Try another start value for sigma.\n");
-		pos_def = 1;
-	    }else{
-		pos_def = 1;
-	    }
-	}
-	F77_CALL(dposl)(hess, &bdim, &true_bdim, db);
-	
-	L1 = 0.0; /*F77_CALL(dnrm2)(&bdim, db, &one);*/
-	for (i = 0; i < true_bdim; i++){
-	    L1 += fabs(db[i]);
-	    beta[i] += db[i];
-	}
-	if (trace)
-	    printf("*** Iteration %d: L1 = %f, loglik = %f\n", iter, 
-		   L1, *loglik);
-	conver = (L1 < epsilon);
-	if (!conver){ 
-	    conver = 
-		fabs(*loglik - fprev) < epsilon;/* * (fabs(fprev) + eps); */
-	}
-
-	if (conver){
-	    if (iter > 0){ /* Always take at least one step! */
-		if (trace){
-		    printf("Newton-Raphson CONVERGENCE in %d step(s)!!\n", 
-			   iter);
-		}
-		break;
-	    }
-	}
-	fprev = *loglik;
-	fun2(bdim, beta, loglik, gr, hess, ex);
-	if (*loglik < fprev){
-	    Rprintf("Warning: Decreasing loglik!!!!!!!!!!!!!!!!!!!!!!!\n");
-	}
-    }
-
-    /* Note: __packed__ form (should use pos. definite versions?) */
-    /* F77_CALL(dpptrf)(&u, &(bdim), hess, &info); */
-
-    for ( m = 0; m < bdim * bdim; m++){
-	c_hess[m] = hess[m];
-    }
-    F77_CALL(dpoco)(hess, &bdim, &true_bdim, &rcond, work, &info);
-    if (info == 0){
-	F77_CALL(dpodi)(hess, &bdim, &true_bdim, det, &job);
-
-	for (m = 0; m < bdim; m++){
-	    for (k = 0; k < m; k++){
-		hess[m + k * bdim] = hess[k + m * bdim];
-	    }
-	}
-    }else{
-	Rprintf("info = %d\n", info);
-	for ( m = 0; m < bdim * bdim; m++){
-	    hess[m] = c_hess[m];
-	}
-	warning("Hessian non-positive definite. No variance!");
-    }
-
-    Free(c_hess);
-    Free(work);
-    Free(ipiv);
-    Free(db);
-    Free(det);
-}
